searchData={"items":[{"type":"module","title":"Telemetrex","doc":"Elixir wrapper for `:telemetry`.\n\nWith the `:telemetry` library being written in a way that was both Erlang and\nElixir friendly, the syntax can be heavy for adding telemetry to business rules.\nFor example, given the following module:\n\n```elixir\ndefmodule MyApp.Users do\n  def user_create(user_params) do\n    %MyApp.User{}\n    |> MyApp.User.create_changeset(users_params)\n    |> MyApp.Repo.insert()\n  end\nend\n```\n\nIf wanting to implement telemetry using `:telemetry.span`, it would result in\nthe following:\n\n```elixir\ndefmodule MyApp.Users do\n  def user_create(user_params) do\n    :telemetry.span([:my_app, :user_create], %{params: user_params}, fn -> \n      %MyApp.User{}\n      |> MyApp.User.create_changeset(users_params)\n      |> MyApp.Repo.insert() \n      |> case do\n        {:ok, %MyApp.User{} = user} = result ->\n          {result, %{user: user}}\n        {:error, changeset} = result ->\n          {result, %{error: true, changeset: changeset}}\n      end\n    end)\n  end\nend\n```\n\nBy adding telemetry with `:telemetry.span`, this operation created a lot of\nadditional noise and was more intrusive to the business rules. By leveraging\nElixir macros, much friendlier syntax can be achieved:\n\n```elixir\ndefmodule MyApp.Users do\n  require Telemetrex\n\n  def user_create(user_params) do\n    Telemetrex.span event: [:my_app, :user_create], context: %{params: user_params} do\n      %MyApp.User{}\n      |> MyApp.User.create_changeset(user_params)\n      |> MyApp.Repo.insert()\n    after\n      {:ok, %MyApp.User{} = user} ->\n        %{user: user}\n\n      {:error, changeset} ->\n        %{error: true, changeset: changeset}\n    end\n  end\nend\n```","ref":"Telemetrex.html"},{"type":"macro","title":"Telemetrex.span/2","doc":"An Elixir-friendly wrapper for `:telemetry.span`.\n\nEverything in the `do` block becomes the return value of the macro. This\nvalue is also passed to the optional `after` block, which can be used to set\nmetadata for the ending event.\n\n> Similar to `:telemetry.span`, the metadata in the ending event is not\nmerged with the initial context. If you want them merged, manually do so in\nthe after block clause.","ref":"Telemetrex.html#span/2"},{"type":"extras","title":"Testing Guidelines","doc":"# Testing Guidelines\n\nAs Telemetrex is an Elixir-friendly wrapper around `:telemetry`, it is advised to follow `:telemetry` best practices with regards to testing as well.\n\n[:telemetry_test](https://hexdocs.pm/telemetry/telemetry_test.html)\n\nGiven the following module implementing `Telemetrex.span`:\n\n```elixir\ndefmodule MyApp.Users do\n  require Telemetrex\n\n  def user_create(user_params) do\n    Telemetrex.span event: [:my_app, :user_create], context: %{params: user_params} do\n      %MyApp.User{}\n      |> MyApp.User.create_changeset(user_params)\n      |> MyApp.Repo.insert()\n    after\n      {:ok, %MyApp.User{} = user} ->\n        %{user: user}\n\n      {:error, changeset} ->\n        %{error: true, changeset: changeset}\n    end\n  end\nend\n```\n\nHere's how we could test the firing of the events:\n\n```elixir\ntest \"test events are fired\" do\n  # Arrange\n  ref = :telemetry_test.attach_event_handlers(self(), [[:my_app, :user_create, :start], [:my_app, :user_create, :start]])\n\n  # Act\n  MyApp.Users.user_create(%{first_name: \"thanks\", last_name: \"telemetry\"})\n\n  # Assert\n  assert_received {[:my_app, :user_create, :start], ^ref, %{measurement: _}, %{meta: _}}\n  assert_received {[:my_app, :user_create, :stop], ^ref, %{measurement: _}, %{meta: _}}\nend\n```","ref":"testing.html"}],"content_type":"text/markdown"}